<?php

namespace Drupal\web_host_metadata\Controller;

use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\web_host_metadata\Event\WebHostMetadataEvents;
use Drupal\web_host_metadata\Event\WebHostMetadataResponseEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for Web Host Metadata routes.
 */
class WebHostMetadataController extends ControllerBase {

  const CONTENT_TYPE_JSON = 'application/jrd+json';

  const CONTENT_TYPE_XML = 'application/xrd+xml; charset=utf-8';

  /**
   * The event dispatcher.
   *
   * @var \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher
   */
  protected ContainerAwareEventDispatcher $eventDispatcher;

  /**
   * The controller constructor.
   *
   * @param \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(ContainerAwareEventDispatcher $event_dispatcher) {
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('event_dispatcher')
    );
  }

  /**
   * Builds the response.
   *
   * @param string $format
   *   The format string e.g. xml, json.
   * @param string $content_type
   *   The response content-type string.
   *
   * @return \Drupal\Core\Cache\CacheableResponse
   *   The cacheable response.
   */
  private function build(string $format, string $content_type) {
    $response_cacheability = new CacheableMetadata();
    $url = Url::fromRoute('web_host_metadata.host_meta', [], ['absolute' => TRUE])->toString(TRUE);
    $response_cacheability->addCacheableDependency($url);

    $xml_xrd = new \XML_XRD();
    $event = new WebHostMetadataResponseEvent($xml_xrd, $response_cacheability);
    $this->eventDispatcher->dispatch($event, WebHostMetadataEvents::RESPONSE);

    $response = new CacheableResponse($xml_xrd->to($format), 200, ['content-type' => $content_type]);
    $response->addCacheableDependency($response_cacheability);
    return $response;
  }

  /**
   * Builds the XML response.
   *
   * @return \Drupal\Core\Cache\CacheableResponse
   *   The cacheable response.
   */
  public function xml(): CacheableResponse {
    return $this->build('xml', static::CONTENT_TYPE_XML);
  }

  /**
   * Builds the JSON response.
   *
   * @return \Drupal\Core\Cache\CacheableResponse
   *   The cacheable response.
   */
  public function json(): CacheableResponse {
    return $this->build('json', static::CONTENT_TYPE_JSON);
  }

}
